
#define WINDOW_WIDTH  				131
#define WINDOW_HEIGHT  				80
#define NULL 						0
#define PI         					((float) 3.141592654)
#define DEG_TO_RAD(ang) 			((ang) * PI / 180.0)
#define EPSILON_E5 					(float) (1E-5)
#define BACKFACE     				0x0001 
#define MD2_ID       				(('I') + ('D' << 8) + ('P' << 16) + ('2' << 24))
#define MD2_VERSION        			8 

typedef struct {
	float x,y,z;
} VECTOR3D, POINT3D, *VECTOR3D_PTR, *POINT3D_PTR;

typedef struct {
	POINT3D p0; 									
	VECTOR3D n; 									
} PLANE, *PLANE_PTR;

typedef struct {
	union {
		float M[4][4];
		struct {
			float M00, M01, M02, M03;
			float M10, M11, M12, M13;
			float M20, M21, M22, M23;
			float M30, M31, M32, M33;
		};
	};
} MATRICE, *MATRICE_PTR;

typedef struct {
	union {
		float M[4]; 
		struct { float x,y,z,w;};
	};
} VECTOR4D, POINT4D, *VECTOR4D_PTR, *POINT4D_PTR;

typedef struct {
	union {     
		struct {float x,y,z,w;};    
		struct  {POINT4D  v;};             
	}; 
} VERTEX, *VERTEX_PTR;

typedef struct {					 
	int backface;
	VERTEX_PTR vlist;
	int vert[3];
} POLYGONE, *POLYGONE_PTR;

typedef struct {
	int state;   
	POINT4D world_pos; 
	int num_vertices;    
	VERTEX_PTR vlist;
	int num_polys;   
	POLYGONE_PTR plist;  
	POLYGONE_PTR *pplist;  
} OBJECT, *OBJECT_PTR;

typedef struct {   					
	POINT4D pos;    								
	VECTOR4D dir;  	 							
	float view_dist;  								
	float fov;        							
	float near_z;    							
	float far_z;      					
	PLANE rt_plane;  					
	PLANE lt_plane;  						
	PLANE tp_plane;  						
	PLANE bt_plane;  						              
	float viewplane_width;     					
	float viewplane_height;  					  	
	float viewport_width; 						   
	float viewport_height;
	float viewport_center_x;  						
	float viewport_center_y;
	float aspect_ratio;
	MATRICE mcam;   								
	MATRICE mper; 							    
	MATRICE mscr; 							    
} CAMERA, *CAMERA_PTR;

typedef struct {
	int identifier;     						
	int version;        					
	int skin_width;     						
	int skin_height;    						
	int framesize;      						
	int num_skins;      						
	int num_verts;      					
	int num_textcoords; 						
	int num_polys;      						
	int num_openGLcmds; 						
	int num_frames;     											
	int offset_skins;      					
	int offset_textcoords; 						
	int offset_polys;      						
	int offset_frames;     						
	int offset_openGLcmds; 						
	int offset_end;						        
} MD2_HEADER; 

typedef struct {
	unsigned char v[3];          				
	unsigned char normal_index;  				
} MD2_POINT; 

typedef struct {
	float scale[3];          					
	float translate[3];      					
	char name[16];           					
} MD2_FRAME;

typedef struct {
	unsigned short vindex[3];   			
} MD2_POLYGONE, *MD2_POLYGONE_PTR;

typedef struct {
	MD2_POLYGONE_PTR plist; 						
	VECTOR3D_PTR vlist; 						
} MD2_CONTAINER; 

typedef struct {
	int buffer_capacity;
	int num_frames_inbuffer;
	float frame_played_inbuffer;
	int frame_first_infile;						
} FRAME_BUFFER;
	
const MATRICE ID_MAT = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};

#define MAT_IDENTITY(m) {memcpy((void *)(m), (void *)&ID_MAT, sizeof(MATRICE));}
#define MAT_COPY(src_mat, dest_mat) {memcpy((void *)(dest_mat), (void *)(src_mat), sizeof(MATRICE) ); }
#define SWAP(a,b,t) {t=a; a=b; b=t;}

int compare_polygone(const void *arg1, const void *arg2);
void init_plane(PLANE_PTR plane, POINT3D_PTR p0, VECTOR3D_PTR normal);
void init_matrice(MATRICE_PTR ma, 
				float m00, float m01, float m02, float m03, float m10, float m11, float m12, float m13,
				float m20, float m21, float m22, float m23, float m30, float m31, float m32, float m33);				
				
void multiply_matrice(MATRICE_PTR ma, MATRICE_PTR mb, MATRICE_PTR mprod);
void build_rotation_matrice(float theta_x, float theta_y, float theta_z, MATRICE_PTR mrot);
void Init_CAM4DV1(CAMERA_PTR cam, POINT4D_PTR cam_pos, VECTOR4D_PTR cam_dir, float near_z, float far_z, 
					float fov, float viewport_width,  float viewport_height);
void build_camera(CAMERA_PTR cam);
int load_md2file(char *md2name);








